@@ -1,14 +1,15 @@
 use core::{
     borrow::{Borrow, BorrowMut},
     cmp::Ordering,
+    convert::Infallible,
     fmt,
     hash::{Hash, Hasher},
-    iter::FromIterator,
+    iter::{FromIterator, FusedIterator},
     ops, ptr, slice,
     str::{self, Chars, Utf8Error},
 };
 
-use alloc::{borrow::Cow, boxed::Box, string::String};
+use alloc::{borrow::Cow, boxed::Box, str::FromStr, string::String};
 
 #[cfg(feature = "ffi")]
 use std::ffi::{OsStr, OsString};
@@ -52,6 +53,7 @@ impl<A: Array<Item = u8>> SmallString<A> {
     }
 
     /// Construct a `SmallString` by copying data from a `&str`.
+    #[allow(clippy::should_implement_trait)]
     #[inline]
     pub fn from_str(s: &str) -> SmallString<A> {
         SmallString {
@@ -131,7 +133,7 @@ impl<A: Array<Item = u8>> SmallString<A> {
     }
 
     /// Empties the string and returns an iterator over its former contents.
-    pub fn drain(&mut self) -> Drain {
+    pub fn drain(&mut self) -> Drain<'_> {
         unsafe {
             let len = self.len();
 
@@ -146,6 +148,29 @@ impl<A: Array<Item = u8>> SmallString<A> {
         }
     }
 
+    /// Empties the sub string and returns an iterator over its former contents.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use smallstr::SmallString;
+    ///
+    /// let mut s: SmallString<[u8; 8]> = SmallString::from("foo bar");
+    /// let sub: String = s.drain_range(1..5).collect();
+    ///
+    /// assert_eq!(sub, "oo b");
+    /// assert_eq!(s, "far");
+    /// ```
+    #[doc(alias = "drain")]
+    pub fn drain_range<R>(&mut self, range: R) -> DrainRange<'_, A>
+    where
+        R: ops::RangeBounds<usize>,
+    {
+        DrainRange {
+            drain: self.data.drain(range),
+        }
+    }
+
     /// Appends the given `char` to the end of this string.
     ///
     /// # Examples
@@ -285,11 +310,9 @@ impl<A: Array<Item = u8>> SmallString<A> {
         let len = self.len();
 
         unsafe {
-            ptr::copy(
-                self.as_ptr().add(next),
-                self.as_mut_ptr().add(idx),
-                len - next,
-            );
+            let ptr = self.as_mut_ptr();
+
+            ptr::copy(ptr.add(next), ptr.add(idx), len - next);
             self.data.set_len(len - ch_len);
         }
 
@@ -325,13 +348,11 @@ impl<A: Array<Item = u8>> SmallString<A> {
 
         self.data.reserve(amt);
 
+        let ptr = self.as_mut_ptr();
+
         unsafe {
-            ptr::copy(
-                self.as_ptr().add(idx),
-                self.as_mut_ptr().add(idx + amt),
-                len - idx,
-            );
-            ptr::copy_nonoverlapping(s.as_ptr(), self.as_mut_ptr().add(idx), amt);
+            ptr::copy(ptr.add(idx), ptr.add(idx + amt), len - idx);
+            ptr::copy_nonoverlapping(s.as_ptr(), ptr.add(idx), amt);
             self.data.set_len(len + amt);
         }
     }
@@ -429,10 +450,12 @@ impl<A: Array<Item = u8>> SmallString<A> {
             if !f(ch) {
                 guard.del_bytes += ch_len;
             } else if guard.del_bytes > 0 {
+                let ptr = guard.s.as_mut_ptr();
+
                 unsafe {
                     ptr::copy(
-                        guard.s.data.as_ptr().add(guard.idx),
-                        guard.s.data.as_mut_ptr().add(guard.idx - guard.del_bytes),
+                        ptr.add(guard.idx),
+                        ptr.add(guard.idx - guard.del_bytes),
                         ch_len,
                     );
                 }
@@ -445,8 +468,12 @@ impl<A: Array<Item = u8>> SmallString<A> {
         drop(guard);
     }
 
+    fn as_ptr(&mut self) -> *const u8 {
+        self.data.as_ptr()
+    }
+
     fn as_mut_ptr(&mut self) -> *mut u8 {
-        self.as_ptr() as *mut u8
+        self.data.as_mut_ptr()
     }
 }
 
@@ -503,13 +530,6 @@ impl<A: Array<Item = u8>> AsRef<[u8]> for SmallString<A> {
     }
 }
 
-impl<A: Array<Item = u8>> Borrow<[u8]> for SmallString<A> {
-    #[inline]
-    fn borrow(&self) -> &[u8] {
-        self.data.borrow()
-    }
-}
-
 impl<A: Array<Item = u8>> fmt::Write for SmallString<A> {
     #[inline]
     fn write_str(&mut self, s: &str) -> fmt::Result {
@@ -524,6 +544,14 @@ impl<A: Array<Item = u8>> fmt::Write for SmallString<A> {
     }
 }
 
+impl<A: Array<Item = u8>> FromStr for SmallString<A> {
+    type Err = Infallible;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        Ok(SmallString::from(s))
+    }
+}
+
 #[cfg(feature = "serde")]
 impl<A: Array<Item = u8>> Serialize for SmallString<A> {
     fn serialize<S: Serializer>(&self, serializer: S) -> Result<S::Ok, S::Error> {
@@ -569,7 +597,7 @@ impl<A: Array<Item = u8>> From<char> for SmallString<A> {
     }
 }
 
-impl<'a, A: Array<Item = u8>> From<&'a str> for SmallString<A> {
+impl<A: Array<Item = u8>> From<&'_ str> for SmallString<A> {
     #[inline]
     fn from(s: &str) -> SmallString<A> {
         SmallString::from_str(s)
@@ -590,6 +618,15 @@ impl<A: Array<Item = u8>> From<String> for SmallString<A> {
     }
 }
 
+impl<'a, A: Array<Item = u8>> From<Cow<'a, str>> for SmallString<A> {
+    fn from(value: Cow<'a, str>) -> Self {
+        match value {
+            Cow::Borrowed(s) => Self::from_str(s),
+            Cow::Owned(s) => Self::from_string(s),
+        }
+    }
+}
+
 macro_rules! impl_index_str {
     ($index_type: ty) => {
         impl<A: Array<Item = u8>> ops::Index<$index_type> for SmallString<A> {
@@ -717,12 +754,7 @@ macro_rules! eq_str {
         impl<'a, A: Array<Item = u8>> PartialEq<$rhs> for SmallString<A> {
             #[inline]
             fn eq(&self, rhs: &$rhs) -> bool {
-                &self[..] == &rhs[..]
-            }
-
-            #[inline]
-            fn ne(&self, rhs: &$rhs) -> bool {
-                &self[..] != &rhs[..]
+                self[..] == rhs[..]
             }
         }
     };
@@ -739,11 +771,6 @@ impl<A: Array<Item = u8>> PartialEq<OsStr> for SmallString<A> {
     fn eq(&self, rhs: &OsStr) -> bool {
         &self[..] == rhs
     }
-
-    #[inline]
-    fn ne(&self, rhs: &OsStr) -> bool {
-        &self[..] != rhs
-    }
 }
 
 #[cfg(feature = "ffi")]
@@ -752,11 +779,6 @@ impl<'a, A: Array<Item = u8>> PartialEq<&'a OsStr> for SmallString<A> {
     fn eq(&self, rhs: &&OsStr) -> bool {
         &self[..] == *rhs
     }
-
-    #[inline]
-    fn ne(&self, rhs: &&OsStr) -> bool {
-        &self[..] != *rhs
-    }
 }
 
 #[cfg(feature = "ffi")]
@@ -765,11 +787,6 @@ impl<A: Array<Item = u8>> PartialEq<OsString> for SmallString<A> {
     fn eq(&self, rhs: &OsString) -> bool {
         &self[..] == rhs
     }
-
-    #[inline]
-    fn ne(&self, rhs: &OsString) -> bool {
-        &self[..] != rhs
-    }
 }
 
 #[cfg(feature = "ffi")]
@@ -778,11 +795,6 @@ impl<'a, A: Array<Item = u8>> PartialEq<Cow<'a, OsStr>> for SmallString<A> {
     fn eq(&self, rhs: &Cow<OsStr>) -> bool {
         self[..] == **rhs
     }
-
-    #[inline]
-    fn ne(&self, rhs: &Cow<OsStr>) -> bool {
-        self[..] != **rhs
-    }
 }
 
 impl<A, B> PartialEq<SmallString<B>> for SmallString<A>
@@ -792,12 +804,7 @@ where
 {
     #[inline]
     fn eq(&self, rhs: &SmallString<B>) -> bool {
-        &self[..] == &rhs[..]
-    }
-
-    #[inline]
-    fn ne(&self, rhs: &SmallString<B>) -> bool {
-        &self[..] != &rhs[..]
+        self[..] == rhs[..]
     }
 }
 
@@ -806,7 +813,7 @@ impl<A: Array<Item = u8>> Eq for SmallString<A> {}
 impl<A: Array<Item = u8>> PartialOrd for SmallString<A> {
     #[inline]
     fn partial_cmp(&self, rhs: &SmallString<A>) -> Option<Ordering> {
-        self[..].partial_cmp(&rhs[..])
+        Some(self.cmp(rhs))
     }
 }
 
@@ -855,6 +862,65 @@ impl<'a> DoubleEndedIterator for Drain<'a> {
     }
 }
 
+/// A range draining iterator for `SmallString`.
+///
+/// This struct is created by the [`drain_range`] method on [`SmallString`].
+///
+/// [`drain_range`]: struct.SmallString.html#method.drain_range
+/// [`SmallString`]: struct.SmallString.html
+pub struct DrainRange<'a, A: Array<Item = u8>> {
+    drain: smallvec::Drain<'a, A>,
+}
+
+impl<A: Array<Item = u8>> fmt::Debug for DrainRange<'_, A> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.drain.fmt(f)
+    }
+}
+
+impl<A: Array<Item = u8>> Iterator for DrainRange<'_, A> {
+    type Item = char;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let mut buf = [0; 4];
+
+        buf[0] = self.drain.next()?;
+        let utf8_len = 1.max(buf[0].leading_ones() as usize);
+
+        for b in &mut buf[1..utf8_len] {
+            *b = self.drain.next().unwrap();
+        }
+
+        unsafe { str::from_utf8_unchecked(&buf[..utf8_len]) }
+            .chars()
+            .next()
+    }
+
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        let len = self.drain.len();
+        (len.div_ceil(4), Some(len))
+    }
+}
+
+impl<A: Array<Item = u8>> DoubleEndedIterator for DrainRange<'_, A> {
+    fn next_back(&mut self) -> Option<Self::Item> {
+        let mut buf = [0; 4];
+        let mut i = 3;
+
+        buf[i] = self.drain.next_back()?;
+
+        while buf[i].leading_ones() == 1 {
+            i -= 1;
+            buf[i] = self.drain.next_back().unwrap();
+        }
+
+        unsafe { str::from_utf8_unchecked(&buf[i..]) }
+            .chars()
+            .next()
+    }
+}
+impl<A: Array<Item = u8>> FusedIterator for DrainRange<'_, A> {}
+
 /// A possible error value when creating a `SmallString` from a byte array.
 ///
 /// This type is the error type for the [`from_buf`] method on [`SmallString`].
@@ -899,6 +965,7 @@ impl<A: Array<Item = u8>> fmt::Display for FromUtf8Error<A> {
 mod test {
     use alloc::{
         borrow::{Cow, ToOwned},
+        str::FromStr,
         string::{String, ToString},
     };
 
@@ -921,6 +988,31 @@ mod test {
         assert!(s.is_empty());
     }
 
+    #[test]
+    fn test_drain_range() {
+        let mut s: SmallString<[u8; 2]> = SmallString::new();
+
+        s.push('a');
+        assert_eq!(s.drain_range(..).collect::<String>(), "a");
+        assert!(s.is_empty());
+
+        s.push_str("xyz");
+
+        assert_eq!(s.drain_range(1..).collect::<String>(), "yz");
+        assert_eq!(s, "x");
+
+        s.push_str("yz");
+
+        assert_eq!(s.drain_range(..2).collect::<String>(), "xy");
+        assert_eq!(s, "z");
+
+        s.clear();
+        s.push_str("测试文本");
+
+        assert_eq!(s.drain_range(3..9).collect::<String>(), "试文");
+        assert_eq!(s, "测本");
+    }
+
     #[test]
     fn test_drain_rev() {
         let mut s: SmallString<[u8; 2]> = SmallString::new();
@@ -938,6 +1030,37 @@ mod test {
         assert!(s.is_empty());
     }
 
+    #[test]
+    fn test_drain_range_rev() {
+        let mut s: SmallString<[u8; 2]> = SmallString::new();
+
+        s.push('a');
+        assert_eq!(s.drain_range(..).rev().collect::<String>(), "a");
+        assert!(s.is_empty());
+
+        // spilling the vec
+        s.push_str("xyz");
+
+        assert_eq!(s.drain_range(..).rev().collect::<String>(), "zyx");
+        assert!(s.is_empty());
+
+        s.push_str("xyz");
+
+        assert_eq!(s.drain_range(1..).rev().collect::<String>(), "zy");
+        assert_eq!(s, "x");
+
+        s.push_str("yz");
+
+        assert_eq!(s.drain_range(..2).rev().collect::<String>(), "yx");
+        assert_eq!(s, "z");
+
+        s.clear();
+        s.push_str("测试文本");
+
+        assert_eq!(s.drain_range(3..9).rev().collect::<String>(), "文试");
+        assert_eq!(s, "测本");
+    }
+
     #[test]
     fn test_eq() {
         let s: SmallString<[u8; 4]> = SmallString::from("foo");
@@ -1102,6 +1225,13 @@ mod test {
         assert_eq!(s, "foobar");
     }
 
+    #[test]
+    fn test_from_str() {
+        let s: SmallString<[u8; 8]> = FromStr::from_str("foo").unwrap();
+
+        assert_eq!(s, "foo");
+    }
+
     #[cfg(feature = "serde")]
     #[test]
     fn test_serde() {
